class PyDataCredentialsError(ValueError):
    """
    Raised when invalid credentials are provided, or tokens have expired.
    """


class PyDataConnectionError(RuntimeError):
    """
    Raised when unable to fetch credentials due to connection error.
    """



"""Private module for fetching Google API credentials."""

import logging

import google.auth
import google.auth.exceptions
import google.oauth2.credentials
from google_auth_oauthlib import flow
import oauthlib.oauth2.rfc6749.errors
import google.auth.transport.requests

from pydata_google_auth import exceptions
from pydata_google_auth import cache
from pydata_google_auth import _webserver


logger = logging.getLogger(__name__)

CLIENT_ID = "262006177488-3425ks60hkk80fssi9vpohv88g6q1iqd.apps.googleusercontent.com"
CLIENT_SECRET = "JSF-iczmzEgbTR-XK-2xaWAc"
GOOGLE_AUTH_URI = "https://accounts.google.com/o/oauth2/auth"
GOOGLE_TOKEN_URI = "https://oauth2.googleapis.com/token"


def default(
    scopes,
    client_id=None,
    client_secret=None,
    credentials_cache=cache.READ_WRITE,
    use_local_webserver=False,
    auth_local_webserver=None,
):
    """
    Get credentials and default project for accessing Google APIs.

    This method first attempts to get credentials via the
    :func:`google.auth.default` function. If it is unable to get valid
    credentials, it then attempts to get user account credentials via the
    :func:`pydata_google_auth.get_user_credentials` function.

    Parameters
    ----------
    scopes : list[str]
        A list of scopes to use when authenticating to Google APIs. See the
        `list of OAuth 2.0 scopes for Google APIs
        <https://developers.google.com/identity/protocols/googlescopes>`_.
    client_id : str, optional
        The client secrets to use when prompting for user credentials.
        Defaults to a client ID associated with pydata-google-auth.

        If you are a tool or library author, you must override the default
        value with a client ID associated with your project. Per the `Google
        APIs terms of service <https://developers.google.com/terms/>`_, you
        must not mask your API client's identity when using Google APIs.
    client_secret : str, optional
        The client secrets to use when prompting for user credentials.
        Defaults to a client secret associated with pydata-google-auth.

        If you are a tool or library author, you must override the default
        value with a client secret associated with your project. Per the
        `Google APIs terms of service
        <https://developers.google.com/terms/>`_, you must not mask your API
        client's identity when using Google APIs.
    credentials_cache : pydata_google_auth.cache.CredentialsCache, optional
        An object responsible for loading and saving user credentials.

        By default, pydata-google-auth reads and writes credentials in
        ``$HOME/.config/pydata/pydata_google_credentials.json`` or
        ``$APPDATA/.config/pydata/pydata_google_credentials.json`` on
        Windows.
    use_local_webserver : bool, optional
        Use a local webserver for the user authentication
        :class:`google_auth_oauthlib.flow.InstalledAppFlow`. Binds a
        webserver to an open port on ``localhost`` between 8080 and 8089,
        inclusive, to receive authentication token. If not set, defaults to
        ``False``, which requests a token via the console.
    auth_local_webserver : deprecated
        Use the ``use_local_webserver`` parameter instead.

    Returns
    -------
    credentials, project_id : tuple[google.auth.credentials.Credentials, str or None]
        credentials : OAuth 2.0 credentials for accessing Google APIs

        project_id : A default Google developer project ID, if one could be determined
        from the credentials. For example, this returns the project ID
        associated with a service account when using a service account key
        file. It returns None when using user-based credentials.

    Raises
    ------
    pydata_google_auth.exceptions.PyDataCredentialsError
        If unable to get valid credentials.
    """
    if auth_local_webserver is not None:
        use_local_webserver = auth_local_webserver

    # Try to retrieve Application Default Credentials
    credentials, default_project = get_application_default_credentials(scopes)

    if credentials and credentials.valid:
        return credentials, default_project

    credentials = get_user_credentials(
        scopes,
        client_id=client_id,
        client_secret=client_secret,
        credentials_cache=credentials_cache,
        use_local_webserver=use_local_webserver,
    )

    if not credentials or not credentials.valid:
        raise exceptions.PyDataCredentialsError("Could not get any valid credentials.")

    return credentials, None


def get_application_default_credentials(scopes):
    """
    This method tries to retrieve the "default application credentials".
    This could be useful for running code on Google Cloud Platform.

    Parameters
    ----------
    project_id (str, optional): Override the default project ID.

    Returns
    -------
    - GoogleCredentials,
        If the default application credentials can be retrieved
        from the environment. The retrieved credentials should also
        have access to the project (project_id) on BigQuery.
    - OR None,
        If default application credentials can not be retrieved
        from the environment. Or, the retrieved credentials do not
        have access to the project (project_id) on BigQuery.
    """

    try:
        credentials, project = google.auth.default(scopes=scopes)
    except (google.auth.exceptions.DefaultCredentialsError, IOError) as exc:
        logger.debug("Error getting default credentials: {}".format(str(exc)))
        return None, None

    if credentials and not credentials.valid:
        request = google.auth.transport.requests.Request()
        try:
            credentials.refresh(request)
        except google.auth.exceptions.RefreshError:
            # Sometimes (such as on Travis) google-auth returns GCE
            # credentials, but fetching the token for those credentials doesn't
            # actually work. See:
            # https://github.com/googleapis/google-auth-library-python/issues/287
            return None, None

    return credentials, project


def get_user_credentials(
    scopes,
    client_id=None,
    client_secret=None,
    credentials_cache=cache.READ_WRITE,
    use_local_webserver=False,
    auth_local_webserver=None,
):
    """
    Gets user account credentials.

    This function authenticates using user credentials, either loading saved
    credentials from the cache or by going through the OAuth 2.0 flow.

    The default read-write cache attempts to read credentials from a file on
    disk. If these credentials are not found or are invalid, it begins an
    OAuth 2.0 flow to get credentials. You'll open a browser window asking
    for you to authenticate to your Google account using the product name
    ``PyData Google Auth``. The permissions it requests correspond to the
    scopes you've provided.

    Additional information on the user credentails authentication mechanism
    can be found `here
    <https://developers.google.com/identity/protocols/OAuth2#clientside/>`__.

    Parameters
    ----------
    scopes : list[str]
        A list of scopes to use when authenticating to Google APIs. See the
        `list of OAuth 2.0 scopes for Google APIs
        <https://developers.google.com/identity/protocols/googlescopes>`_.
    client_id : str, optional
        The client secrets to use when prompting for user credentials.
        Defaults to a client ID associated with pydata-google-auth.

        If you are a tool or library author, you must override the default
        value with a client ID associated with your project. Per the `Google
        APIs terms of service <https://developers.google.com/terms/>`_, you
        must not mask your API client's identity when using Google APIs.
    client_secret : str, optional
        The client secrets to use when prompting for user credentials.
        Defaults to a client secret associated with pydata-google-auth.

        If you are a tool or library author, you must override the default
        value with a client secret associated with your project. Per the
        `Google APIs terms of service
        <https://developers.google.com/terms/>`_, you must not mask your API
        client's identity when using Google APIs.
    credentials_cache : pydata_google_auth.cache.CredentialsCache, optional
        An object responsible for loading and saving user credentials.

        By default, pydata-google-auth reads and writes credentials in
        ``$HOME/.config/pydata/pydata_google_credentials.json`` or
        ``$APPDATA/.config/pydata/pydata_google_credentials.json`` on
        Windows.
    use_local_webserver : bool, optional
        Use a local webserver for the user authentication
        :class:`google_auth_oauthlib.flow.InstalledAppFlow`. Binds a
        webserver to an open port on ``localhost`` between 8080 and 8089,
        inclusive, to receive authentication token. If not set, defaults to
        ``False``, which requests a token via the console.
    auth_local_webserver : deprecated
        Use the ``use_local_webserver`` parameter instead.

    Returns
    -------
    credentials : google.oauth2.credentials.Credentials
        Credentials for the user, with the requested scopes.

    Raises
    ------
    pydata_google_auth.exceptions.PyDataCredentialsError
        If unable to get valid user credentials.
    """
    if auth_local_webserver is not None:
        use_local_webserver = auth_local_webserver

    # Use None as default for client_id and client_secret so that the values
    # aren't included in the docs. A string of bytes isn't useful for the
    # documentation and might encourage the values to be used outside of this
    # library.
    if client_id is None:
        client_id = CLIENT_ID
    if client_secret is None:
        client_secret = CLIENT_SECRET

    credentials = credentials_cache.load()

    client_config = {
        "installed": {
            "client_id": client_id,
            "client_secret": client_secret,
            "redirect_uris": ["urn:ietf:wg:oauth:2.0:oob"],
            "auth_uri": GOOGLE_AUTH_URI,
            "token_uri": GOOGLE_TOKEN_URI,
        }
    }

    if credentials is None:
        app_flow = flow.InstalledAppFlow.from_client_config(
            client_config, scopes=scopes
        )

        try:
            if use_local_webserver:
                credentials = _webserver.run_local_server(app_flow)
            else:
                credentials = app_flow.run_console()
        except oauthlib.oauth2.rfc6749.errors.OAuth2Error as exc:
            raise exceptions.PyDataCredentialsError(
                "Unable to get valid credentials: {}".format(exc)
            )

        credentials_cache.save(credentials)

    if credentials and not credentials.valid:
        request = google.auth.transport.requests.Request()
        credentials.refresh(request)

    return credentials


def save_user_credentials(
    scopes, path, client_id=None, client_secret=None, use_local_webserver=False
):
    """
    Gets user account credentials and saves them to a JSON file at ``path``.

    This function authenticates using user credentials by going through the
    OAuth 2.0 flow.

    Parameters
    ----------

    scopes : list[str]
        A list of scopes to use when authenticating to Google APIs. See the
        `list of OAuth 2.0 scopes for Google APIs
        <https://developers.google.com/identity/protocols/googlescopes>`_.
    path : str
        Path to save credentials JSON file.
    client_id : str, optional
        The client secrets to use when prompting for user credentials.
        Defaults to a client ID associated with pydata-google-auth.

        If you are a tool or library author, you must override the default
        value with a client ID associated with your project. Per the `Google
        APIs terms of service <https://developers.google.com/terms/>`_, you
        must not mask your API client's identity when using Google APIs.
    client_secret : str, optional
        The client secrets to use when prompting for user credentials.
        Defaults to a client secret associated with pydata-google-auth.

        If you are a tool or library author, you must override the default
        value with a client secret associated with your project. Per the
        `Google APIs terms of service
        <https://developers.google.com/terms/>`_, you must not mask your API
        client's identity when using Google APIs.
    use_local_webserver : bool, optional
        Use a local webserver for the user authentication
        :class:`google_auth_oauthlib.flow.InstalledAppFlow`. Binds a
        webserver to an open port on ``localhost`` between 8080 and 8089,
        inclusive, to receive authentication token. If not set, defaults to
        ``False``, which requests a token via the console.

    Returns
    -------

    None

    Raises
    ------
    pydata_google_auth.exceptions.PyDataCredentialsError
        If unable to get valid user credentials.

    Examples
    --------

    Get credentials for Google Cloud Platform and save them to
    ``/home/username/keys/google-credentials.json``.

    .. code-block:: python

       pydata_google_auth.save_user_credentials(
           ["https://www.googleapis.com/auth/cloud-platform"],
           "/home/username/keys/google-credentials.json",
           use_local_webserver=True,
       )

    Set the ``GOOGLE_APPLICATION_CREDENTIALS`` environment variable to use
    these credentials with Google Application Default Credentials.

    .. code-block:: bash

       export GOOGLE_APPLICATION_CREDENTIALS='/home/username/keys/google-credentials.json'
    """
    credentials = get_user_credentials(
        scopes,
        client_id=client_id,
        client_secret=client_secret,
        credentials_cache=cache.NOOP,
        use_local_webserver=use_local_webserver,
    )
    cache._save_user_account_credentials(credentials, path)


def load_user_credentials(path):
    """
    Gets user account credentials from JSON file at ``path``.

    Parameters
    ----------
    path : str
        Path to credentials JSON file.

    Returns
    -------

    google.auth.credentials.Credentials

    Raises
    ------
    pydata_google_auth.exceptions.PyDataCredentialsError
        If unable to load user credentials.

    Examples
    --------

    Load credentials and use them to construct a BigQuery client.

    .. code-block:: python

       import pydata_google_auth
       import google.cloud.bigquery

       credentials = pydata_google_auth.load_user_credentials(
           "/home/username/keys/google-credentials.json",
       )
       client = google.cloud.bigquery.BigQueryClient(
           credentials=credentials,
           project="my-project-id"
       )
    """
    credentials = cache._load_user_credentials_from_file(path)
    if not credentials:
        raise exceptions.PyDataCredentialsError("Could not load credentials.")
    return credentials


def load_service_account_credentials(path, scopes=None):
    """
    Gets service account credentials from JSON file at ``path``.

    Parameters
    ----------
    path : str
        Path to credentials JSON file.
    scopes : list[str], optional
        A list of scopes to use when authenticating to Google APIs. See the
        `list of OAuth 2.0 scopes for Google APIs
        <https://developers.google.com/identity/protocols/googlescopes>`_.

    Returns
    -------

    google.oauth2.service_account.Credentials

    Raises
    ------
    pydata_google_auth.exceptions.PyDataCredentialsError
        If unable to load service credentials.

    Examples
    --------

    Load credentials and use them to construct a BigQuery client.

    .. code-block:: python

       import pydata_google_auth
       import google.cloud.bigquery

       credentials = pydata_google_auth.load_service_account_credentials(
           "/home/username/keys/google-service-account-credentials.json",
       )
       client = google.cloud.bigquery.BigQueryClient(
           credentials=credentials,
           project=credentials.project_id
       )
    """

    credentials = cache._load_service_account_credentials_from_file(path, scopes=scopes)
    if not credentials:
        raise exceptions.PyDataCredentialsError("Could not load credentials.")
    return credentials



"""Private module that implements a pydata-google-auth CLI tool."""

import argparse
import sys

from . import auth


LOGIN_HELP = (
    "Login to Google and save user credentials as a JSON file to use as "
    "Application Default Credentials."
)
LOGIN_SCOPES_DEFAULT = "https://www.googleapis.com/auth/cloud-platform"
LOGIN_SCOPES_HELP = (
    "Comma-separated list of scopes (permissions) to request from Google. "
    "See: https://developers.google.com/identity/protocols/googlescopes for "
    "a list of available scopes. Default: {}"
).format(LOGIN_SCOPES_DEFAULT)
LOGIN_CLIENT_ID_HELP_TEMPLATE = (
    "(Optional, but recommended) Client {}. Use this in combination with "
    "the {other} argument to authenticate with an application other than the "
    "default (PyData Auth). This argument is required to use APIs the track "
    "billing and quotas via the application (such as Cloud Vision), rather "
    "than billing the user (such as BigQuery does)."
)
LOGIN_CLIENT_ID_HELP = LOGIN_CLIENT_ID_HELP_TEMPLATE.format(
    "ID", other="--client-secret"
)
LOGIN_CLIENT_SECRET_HELP = LOGIN_CLIENT_ID_HELP_TEMPLATE.format(
    "secret", other="--client-id"
)
LOGIN_USE_LOCAL_WEBSERVER_HELP = (
    "Use a local webserver for the user authentication. This starts "
    "a webserver on localhost with a port between 8080 and 8089, "
    "inclusive, which allows the browser to pass a token directly to the "
    "program."
)

PRINT_TOKEN_HELP = "Load a credentials JSON file and print an access token."
PRINT_TOKEN_DESCRIPTION = r"""examples:

  Download the contents of gs://your-bucket/path/to/object.txt with the Google
  Cloud Storage JSON REST API.

    curl -X GET \
        -H "Authorization: Bearer $(python -m pydata_google_auth print-token credentials.json)" \
        "https://storage.googleapis.com/storage/v1/b/your-bucket/o/path%%2Fto%%2Fobject.txt?alt=media"
"""


def login(args):
    scopes = args.scopes.split(",")
    auth.save_user_credentials(
        scopes,
        args.destination,
        client_id=args.client_id,
        client_secret=args.client_secret,
        use_local_webserver=args.use_local_webserver,
    )


def print_token(args):
    credentials = auth.load_user_credentials(args.credentials_path)
    print(credentials.token)


parser = argparse.ArgumentParser(
    prog="python -m pydata_google_auth",
    description="Manage credentials for Google APIs.",
)
subparsers = parser.add_subparsers(title="commands", dest="command")

login_parser = subparsers.add_parser("login", help=LOGIN_HELP)
login_parser.add_argument(
    "destination", help="Path of where to save user credentials JSON file."
)
login_parser.add_argument(
    "--scopes", help=LOGIN_SCOPES_HELP, default=LOGIN_SCOPES_DEFAULT
)
login_parser.add_argument("--client_id", help=LOGIN_CLIENT_ID_HELP)
login_parser.add_argument("--client_secret", help=LOGIN_CLIENT_SECRET_HELP)
login_parser.add_argument(
    "--use_local_webserver", action="store_true", help=LOGIN_USE_LOCAL_WEBSERVER_HELP
)

print_token_parser = subparsers.add_parser(
    "print-token",
    help=PRINT_TOKEN_HELP,
    description=PRINT_TOKEN_DESCRIPTION,
    formatter_class=argparse.RawDescriptionHelpFormatter,
)
print_token_parser.add_argument(
    "credentials_path", help="Path of credentials JSON file."
)

args = parser.parse_args()
if args.command == "login":
    login(args)
elif args.command == "print-token":
    print_token(args)
else:
    print('Got unknown command "{}".'.format(args.command), file=sys.stderr)
    parser.print_help()



"""Helpers for running a local webserver to receive authorization code."""

import socket
from contextlib import closing

from pydata_google_auth import exceptions


LOCALHOST = "localhost"
DEFAULT_PORTS_TO_TRY = 100


def is_port_open(port):
    """Check if a port is open on localhost.

    Based on StackOverflow answer: https://stackoverflow.com/a/43238489/101923

    Parameters
    ----------
    port : int
        A port to check on localhost.

    Returns
    -------
    is_open : bool
        True if a socket can be opened at the requested port.
    """
    with closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as sock:
        try:
            sock.bind((LOCALHOST, port))
            sock.listen(1)
        except socket.error:
            is_open = False
        else:
            is_open = True
    return is_open


def find_open_port(start=8080, stop=None):
    """Find an open port between ``start`` and ``stop``.

    Parameters
    ----------
    start : Optional[int]
        Beginning of range of ports to try. Defaults to 8080.
    stop : Optional[int]
        End of range of ports to try (not including exactly equals ``stop``).
        This function tries 100 possible ports if no ``stop`` is specified.

    Returns
    -------
    Optional[int]
        ``None`` if no open port is found, otherwise an integer indicating an
        open port.
    """
    if not stop:
        stop = start + DEFAULT_PORTS_TO_TRY

    for port in range(start, stop):
        if is_port_open(port):
            return port

    # No open ports found.
    return None


def run_local_server(app_flow):
    """Run local webserver installed app flow on some open port.

    Parameters
    ----------
    app_flow : google_auth_oauthlib.flow.InstalledAppFlow
        Installed application flow to fetch user credentials.

    Returns
    -------
    google.auth.credentials.Credentials
        User credentials from installed application flow.

    Raises
    ------
    pydata_google_auth.exceptions.PyDataConnectionError
        If no open port can be found in the range from 8080 to 8089,
        inclusive.
    """
    port = find_open_port()
    if not port:
        raise exceptions.PyDataConnectionError("Could not find open port.")
    return app_flow.run_local_server(host=LOCALHOST, port=port)



"""Caching implementations for reading and writing user credentials."""

import errno
import json
import logging
import os
import os.path

import google.oauth2.credentials
from google.oauth2 import service_account


logger = logging.getLogger(__name__)


_DIRNAME = "pydata"
_FILENAME = "pydata_google_credentials.json"


def _get_default_credentials_path(credentials_dirname, credentials_filename):
    """
    Gets the default path to the Google user credentials

    Returns
    -------
    str
        Path to the Google user credentials
    """
    config_path = None

    if os.name == "nt":
        config_path = os.getenv("APPDATA")
    if not config_path:
        config_path = os.path.join(os.path.expanduser("~"), ".config")

    config_path = os.path.join(config_path, credentials_dirname)
    return os.path.join(config_path, credentials_filename)


def _load_user_credentials_from_info(credentials_json):
    credentials = google.oauth2.credentials.Credentials(
        token=credentials_json.get("access_token"),
        refresh_token=credentials_json.get("refresh_token"),
        id_token=credentials_json.get("id_token"),
        token_uri=credentials_json.get("token_uri"),
        client_id=credentials_json.get("client_id"),
        client_secret=credentials_json.get("client_secret"),
        scopes=credentials_json.get("scopes"),
    )

    if credentials and not credentials.valid:
        request = google.auth.transport.requests.Request()
        try:
            credentials.refresh(request)
        except google.auth.exceptions.RefreshError:
            # Credentials could be expired or revoked. Try to reauthorize.
            return None

    return credentials


def _load_user_credentials_from_file(credentials_path):
    """
    Loads user account credentials from a local file.

    Parameters
    ----------
    None

    Returns
    -------
    - GoogleCredentials,
        If the credentials can loaded. The retrieved credentials should
        also have access to the project (project_id) on BigQuery.
    - OR None,
        If credentials can not be loaded from a file. Or, the retrieved
        credentials do not have access to the project (project_id)
        on BigQuery.
    """
    try:
        with open(credentials_path) as credentials_file:
            credentials_json = json.load(credentials_file)
    except (IOError, ValueError) as exc:
        logger.debug(
            "Error loading credentials from {}: {}".format(credentials_path, str(exc))
        )
        return None

    return _load_user_credentials_from_info(credentials_json)


def _save_user_account_credentials(credentials, credentials_path):
    """
    Saves user account credentials to a local file.
    """

    # Create the direcory if it doesn't exist.
    # https://stackoverflow.com/a/12517490/101923
    config_dir = os.path.dirname(credentials_path)
    if not os.path.exists(config_dir):
        try:
            os.makedirs(config_dir)
        except OSError as exc:  # Guard against race condition.
            if exc.errno != errno.EEXIST:
                logger.warning("Unable to create credentials directory.")
                return

    try:
        with open(credentials_path, "w") as credentials_file:
            credentials_json = {
                "refresh_token": credentials.refresh_token,
                "id_token": credentials.id_token,
                "token_uri": credentials.token_uri,
                "client_id": credentials.client_id,
                "client_secret": credentials.client_secret,
                "scopes": credentials.scopes,
                # Required for Application Default Credentials to detect the
                # credentials type. See:
                # https://github.com/pydata/pydata-google-auth/issues/22
                "type": "authorized_user",
            }
            json.dump(credentials_json, credentials_file)
    except IOError:
        logger.warning("Unable to save credentials.")


def _load_service_account_credentials_from_file(credentials_path, **kwargs):
    try:
        with open(credentials_path) as credentials_file:
            credentials_json = json.load(credentials_file)
    except (IOError, ValueError) as exc:
        logger.debug(
            "Error loading credentials from {}: {}".format(credentials_path, str(exc))
        )
        return None

    return _load_service_account_credentials_from_info(credentials_json, **kwargs)


def _load_service_account_credentials_from_info(credentials_json, **kwargs):
    credentials = service_account.Credentials.from_service_account_info(
        credentials_json, **kwargs
    )
    if not credentials.valid:
        request = google.auth.transport.requests.Request()
        try:
            credentials.refresh(request)
        except google.auth.exceptions.RefreshError as exc:
            # Credentials could be expired or revoked.
            logger.debug("Error refreshing credentials: {}".format(str(exc)))
            return None

    return credentials


class CredentialsCache(object):
    """
    Shared base class for crentials classes.

    This class also functions as a noop implementation of a credentials class.
    """

    def load(self):
        """
        Load credentials from disk.

        Does nothing in this base class.

        Returns
        -------
        google.oauth2.credentials.Credentials, optional
            Returns user account credentials loaded from disk or ``None`` if no
            credentials could be found.
        """
        pass

    def save(self, credentials):
        """
        Write credentials to disk.

        Does nothing in this base class.

        Parameters
        ----------
        credentials : google.oauth2.credentials.Credentials
            User credentials to save to disk.
        """
        pass


class ReadWriteCredentialsCache(CredentialsCache):
    """
    A :class:`~pydata_google_auth.cache.CredentialsCache` which writes to
    disk and reads cached credentials from disk.

    Parameters
    ----------
    dirname : str, optional
        Name of directory to write credentials to. This directory is created
        within the ``.config`` subdirectory of the ``HOME`` (``APPDATA`` on
        Windows) directory.
    filename : str, optional
        Name of the credentials file within the credentials directory.
    """

    def __init__(self, dirname=_DIRNAME, filename=_FILENAME):
        super(ReadWriteCredentialsCache, self).__init__()
        self._path = _get_default_credentials_path(dirname, filename)

    def load(self):
        """
        Load credentials from disk.

        Returns
        -------
        google.oauth2.credentials.Credentials, optional
            Returns user account credentials loaded from disk or ``None`` if no
            credentials could be found.
        """
        return _load_user_credentials_from_file(self._path)

    def save(self, credentials):
        """
        Write credentials to disk.

        Parameters
        ----------
        credentials : google.oauth2.credentials.Credentials
            User credentials to save to disk.
        """
        _save_user_account_credentials(credentials, self._path)


class WriteOnlyCredentialsCache(CredentialsCache):
    """
    A :class:`~pydata_google_auth.cache.CredentialsCache` which writes to
    disk, but doesn't read from disk.

    Use this class to reauthorize against Google APIs and cache your
    credentials for later.

    Parameters
    ----------
    dirname : str, optional
        Name of directory to write credentials to. This directory is created
        within the ``.config`` subdirectory of the ``HOME`` (``APPDATA`` on
        Windows) directory.
    filename : str, optional
        Name of the credentials file within the credentials directory.
    """

    def __init__(self, dirname=_DIRNAME, filename=_FILENAME):
        super(WriteOnlyCredentialsCache, self).__init__()
        self._path = _get_default_credentials_path(dirname, filename)

    def save(self, credentials):
        """
        Write credentials to disk.

        Parameters
        ----------
        credentials : google.oauth2.credentials.Credentials
            User credentials to save to disk.
        """
        _save_user_account_credentials(credentials, self._path)


NOOP = CredentialsCache()
"""
Noop impmentation of credentials cache.

This cache always reauthorizes and never save credentials to disk.
Recommended for shared machines.
"""

READ_WRITE = ReadWriteCredentialsCache()
"""
Write credentials to disk and read cached credentials from disk.
"""

REAUTH = WriteOnlyCredentialsCache()
"""
Write credentials to disk. Never read cached credentials from disk.

Use this to reauthenticate and refresh the cached credentials.
"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import versioneer
from setuptools import find_packages, setup

NAME = "pydata-google-auth"


# versioning
cmdclass = versioneer.get_cmdclass()


def readme():
    with open("README.rst") as f:
        return f.read()


INSTALL_REQUIRES = ["setuptools", "google-auth", "google-auth-oauthlib"]

setup(
    name=NAME,
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    description="PyData helpers for authenticating to Google APIs",
    long_description=readme(),
    license="BSD License",
    author="The PyData Development Team",
    author_email="pydata@googlegroups.com",
    url="https://github.com/pydata/pydata-google-auth",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: Science/Research",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Scientific/Engineering",
        "License :: OSI Approved :: BSD License",
    ],
    keywords="data",
    install_requires=INSTALL_REQUIRES,
    packages=find_packages(exclude=["contrib", "docs", "tests*"]),
    test_suite="tests",
)



from .auth import default
from .auth import get_user_credentials
from .auth import load_user_credentials
from .auth import save_user_credentials
from .auth import load_service_account_credentials
from ._version import get_versions

versions = get_versions()
__version__ = versions.get("closest-tag", versions["version"])
__git_revision__ = versions["full-revisionid"]

"""pydata-google-auth

This package provides helpers for fetching Google API credentials.
"""

__all__ = [
    "__version__",
    "__git_revision__",
    "default",
    "get_user_credentials",
    "load_user_credentials",
    "save_user_credentials",
    "load_service_account_credentials",
]
